<?php
/*
  Plugin Name: 	WooCommerce Payment Gateway for Pay@All by Counter Service
  Plugin URI: 	
  Description: 	Provides WooCommerce Payment Gateway for Pay@All by Counter Service.Support woo commerce 2.0.0 and above.
  Version:  		1.0.0
  Author: 			Theeravat Suensilpong
  Author URI: 		http://theeravat.com/
  License: 			commercial
  WC require at least: 2.0.0
  WC tested up to: 3.3.4
 */

add_action('plugins_loaded', 'init_payatall_gateway_class');

function init_payatall_gateway_class() {

    //Check class WC_Payment_Gateway is exists
    if (class_exists("WC_Payment_Gateway")) {

        class WC_Gateway_Payatall extends WC_Payment_Gateway {

            /** @var string Access key for ... */
            var $access_key;

            /**
             * Constructor for the gateway.
             *
             * @access public
             * @return void
             */
            public function __construct() {
                global $woocommerce;

                $this->id = 'payatall';
                $this->method_title = __('Pay@all', 'woocommerce');
                $this->icon = apply_filters('woocommerce_payatall_checkout_icon', plugin_dir_url(__FILE__) . 'images/7-counterservice_logo.jpg');
                $this->has_fields = false;
                $this->payatall_url = 'https://gateway.payatall.com/paynow';
					 $this->class_name = 'WC_Gateway_Payatall';
                $this->notify_url = str_replace('https:', 'http:', add_query_arg('wc-api', $this->class_name, home_url('/')));
                $this->home_url = str_replace('https:', 'http:',  home_url('/'));

                // Load the form fields.
                $this->init_form_fields();

                // Load the settings.
                $this->init_settings();

                // Define user set variables
                // Check compatibility
                if ($woocommerce->version <= 1.6) {
                    $this->merchantid = $this->settings['merchantid']; //Merchant No.
                    $this->title = $this->settings['title'];
                    $this->description = $this->settings['description'];
                    $this->invoice_prefix = $this->settings['invoice_prefix'];
                    $this->return_url = $this->settings['return_url'];
                    $this->test = $this->settings['test'];

                    // Save options
                    add_action('woocommerce_update_options_payment_gateways', array($this, 'process_admin_options'));
                } else {
                    $this->merchantid = $this->get_option('merchantid'); //Merchant No.
                    $this->title = $this->get_option('title');
                    $this->description = $this->get_option('description');
                    $this->invoice_prefix = $this->get_option('invoice_prefix');
                    $this->return_url = $this->get_option('return_url');
                    $this->test = $this->get_option('test');

                    // Save options
                    add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
                }

                // Actions
                add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));

                // Payment listener/API hook
                add_action('woocommerce_api_wc_gateway_' . $this->id, array($this, 'payatall_notification'));
            }

            /**
             * payatall_notification function.
             *
             * @access public
             * @return void
             */
            public function payatall_notification() {
                global $woocommerce;

                //Get data from Pay@All
                //Check it
                $sret = trim($_REQUEST["sret"]);
					 list($order_id,$code) = explode('_',$sret);

                $wc_order = new WC_Order($order_id);

                switch ($code) {
                    case '00' :
						  		if($this->return_url != ''){
									 wp_redirect($this->return_url);
									 exit;
								}

                        break;
                    case '01' :
                        $wc_order->update_status('cancelled', __('Pay@All error. check customer info.: name, email, phone No.', 'woocommerce'));
                        break;
                    case '10' :
                        // payment complete
                        $wc_order->add_order_note(__('Pay@All payment completed', 'woocommerce'));

                        // Mark order complete
                        $wc_order->payment_complete();

                        break;
                }

                wp_redirect($this->get_return_url($wc_order));
                exit;
            }

            /**
             * Initialise Gateway Settings Form Fields
             *
             * @access public
             * @return void
             */
            public function init_form_fields() {
                $this->form_fields = array(
                    'enabled' => array(
                        'title' => __('Enable/Disable', 'woocommerce'),
                        'type' => 'checkbox',
                        'label' => __('Enable Pay@All Checkout', 'woocommerce'),
                        'default' => 'no'
                    ),
                    'merchantid' => array(
                        'title' => __('Merchant ID', 'woocommerce'),
                        'type' => 'text',
                        'css' => 'width:300px',
                        'description' => __('Pay@All merchant id.', 'woocommerce'),
                        'default' => '',
                        'desc_tip' => true,
                    ),
                    'title' => array(
                        'title' => __('Title', 'woocommerce'),
                        'type' => 'text',
                        'css' => 'width:300px',
                        'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
                        'default' => __('Pay@All Payment Gateway', 'woocommerce'),
                        'desc_tip' => true,
                    ),
                    'description' => array(
                        'title' => __('Description', 'woocommerce'),
                        'type' => 'textarea',
                        'default' => __('Pay securely with Pay@All Payment Gateway.', 'woocommerce'),
                        'description' => __('This controls the description which the user sees during checkout.', 'woocommerce'),
                        'desc_tip' => true,
                    ),
						  'invoice_prefix' => array(
                        'title' => __('Invoice Prefix', 'woocommerce'),
                        'type' => 'text',
                        'css' => 'width:300px',
                        'description' => __('This add text in front of woocommerce order id.', 'woocommerce'),
                        'desc_tip' => true,
						  ),
						  'return_url' => array(
                        'title' => __('Return URL', 'woocommerce'),
                        'type' => 'text',
                        'css' => 'width:300px',
                        'description' => __('This add text in front of woocommerce order id.', 'woocommerce'),
                        'desc_tip' => true,
						  ),
                    'test' => array(
                        'title' => __('Test System', 'woocommerce'),
                        'type' => 'checkbox',
                        'label' => __('Enable Pay@All testing', 'woocommerce'),
                        'default' => 'no'
                    ),
                );
            }

            /**
             * Admin Panel Options
             * - Options for bits like 'title' and availability on a country-by-country basis
             *
             * @access public
             * @return void
             */
            public function admin_options() {
                ?>
                <h3><?php _e('Pay@All Checkout', 'woocommerce'); ?></h3>

                <table class="form-table">
                <?php $this->generate_settings_html(); ?>
                </table>
                    <?php
				 }

				 /**
				  * Process the payment and return the result
				  *
				  * @access public
				  * @param int $order_id
				  * @return array
				  */
				 public function process_payment($order_id) {
					  global $woocommerce;

					  $order = new WC_Order($order_id);



					  return array(
							'result' => 'success',
							'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay'))))
					  );
				 }


                /**
                 * Output for the order received page.
                 *
                 * @access public
                 * @return void
                 */
                function receipt_page($order_id) {
                    global $woocommerce;

                    $order = new WC_Order($order_id);


                    echo '<p>' . __('Thank you for your order, please wait or click the button below to pay with Pay@All.', 'woocommerce') . '</p>';

                    $merchantid = absint(ltrim($this->merchantid,"P"));
						  $payatall_url = $this->payatall_url;
						  $invoice_prefix = $this->invoice_prefix;
						  $test_system = $this->test;

                    $invoice = ltrim($order->get_order_number(), '#');
						  if($invoice_prefix != '') $invoice = $invoice_prefix.$invoice;
						  if($test_system!='no') $invoice = 'test_'.$invoice; //test

						  $customer_name = $order->get_billing_first_name().' '.$order->get_billing_last_name();
						  $customer_email = $order->get_billing_email();
						  $customer_phone = $order->get_billing_phone();

                    $item_names = array();

                    if (sizeof($order->get_items()) > 0) {
                        foreach ($order->get_items() as $item) {
                            if ($item['qty']) {
                                $item_names[] = $item['name'] . ' x ' . $item['qty'];
                            }
                        }
                    }

                    $description = implode(",", $item_names);
                    $price = $order->get_total();

						  $homeURL = $this->home_url;
                    $postURL = $this->notify_url;
                    $ipCUS = $_SERVER['REMOTE_ADDR'];

						  $back_url = add_query_arg(array(
						  			'wc-api' => $this->class_name,
									'sret' => $order_id.'_00',
								),$homeURL);
						  $cancel_url = add_query_arg(array(
						  			'wc-api' => $this->class_name,
									'sret' => $order_id.'_01',
								),$homeURL);
						  $confirm_url = add_query_arg(array(
						  			'wc-api' => $this->class_name,
									'sret' => $order_id.'_10',
								),$homeURL);
                    
                            
						// Mark as on-hold 
						$order->update_status('on-hold', __( 'Awaiting Pay@All payment', 'woocommerce' ));

						// Reduce stock levels
						$order->reduce_order_stock();

						// Remove cart
						$woocommerce->cart->empty_cart();


                    echo '
				<form name="sendform" id="sendform" method="post" action="' . $payatall_url . '">
				<input type="hidden" id="mid" name="mid" value="' . $merchantid . '">
				<input type="hidden" name="inv" value="' . $invoice . '">
				<input type="hidden" name="desp" value="'.$description.'"/>
				<input type="hidden" name="amt" value="'.$price.'"/>     
				<input type="hidden" name="curr_type" value="THB"/>     
				<input type="hidden" name="serviceid" value="01"/>     
				<input type="hidden" name="language" value="TH"/>
				<input type="hidden" name="customer_name" value="'.$customer_name.'"/>      
				<input type="hidden" name="customer_email" value="'.$customer_email.'"/>     
				<input type="hidden" name="customer_phone" value="'.$customer_phone.'"/>     
				<input type="hidden" name="resp_url_back" value="'.$back_url.'"/>     
				<input type="hidden" name="resp_url_cancel" value="'.$cancel_url.'"/>      
				<input type="hidden" name="resp_url_confirm" value="'.$confirm_url.'"/>
				<input type="hidden" name="expired_date" value=""/>  
				<input type="hidden" name="message_slip1" value=""/>      
				<input type="hidden" name="message_slip2" value=""/>              
				<input type="submit" class="button alt" id="payatall-payment-button" value="' . __( 'Pay via Pay@All Payment Gateway', 'woocommerce' ) . '" /> 

				</form>
		    	';

						  wc_enqueue_js('
				jQuery("#sendform").submit();
				');
				 }
			}

	  }
 }

    /**
     * Add payatall payment gateway into woocommerce payment gateway
     */
    add_filter('woocommerce_payment_gateways', 'add_payatall_gateway');

    function add_payatall_gateway($methods) {
        $methods[] = 'WC_Gateway_Payatall';
        return $methods;
    }
